@@ -33,6 +33,7 @@ our @EXPORT= qw(collect_option collect_test_cases);
 
 use mtr_report;
 use mtr_match;
+use My::Constants;
 
 # Options used for the collect phase
 our $start_from;
@@ -655,6 +656,7 @@ sub optimize_cases {
 	if ( !$supported )
 	{
 	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
 	  $tinfo->{'comment'}=
 	    "Doesn't support --binlog-format='$binlog_format'";
 	}
@@ -681,6 +683,7 @@ sub optimize_cases {
 	if ( !$supported )
 	{
 	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
 	  $tinfo->{'comment'}=
 	    "Doesn't support --binlog-format='$test_binlog_format'";
 	  next;
@@ -716,6 +719,7 @@ sub optimize_cases {
 	     ! exists $builtin_engines{$default_engine} )
 	{
 	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
 	  $tinfo->{'comment'}=
 	    "'$default_engine' not supported";
 	}
@@ -737,6 +741,7 @@ sub optimize_cases {
 	     ! exists $builtin_engines{$default_tmp_engine} )
 	{
 	  $tinfo->{'skip'}= 1;
+	  $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
 	  $tinfo->{'comment'}=
 	    "'$default_tmp_engine' not supported";
 	}
@@ -975,6 +980,7 @@ sub collect_one_test_case {
     if ( IS_WIN32PERL )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "No tests with sh scripts on Windows";
       return $tinfo;
     }
@@ -993,6 +999,7 @@ sub collect_one_test_case {
     if ( IS_WIN32PERL )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "No tests with sh scripts on Windows";
       return $tinfo;
     }
@@ -1026,6 +1033,7 @@ sub collect_one_test_case {
   if ( $tinfo->{'big_test'} and ! $::opt_big_test )
   {
     $tinfo->{'skip'}= 1;
+    $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
     $tinfo->{'comment'}= "Test needs 'big-test' option";
     return $tinfo
   }
@@ -1033,6 +1041,7 @@ sub collect_one_test_case {
   if ( $tinfo->{'need_debug'} && ! $::debug_compiled_binaries )
   {
     $tinfo->{'skip'}= 1;
+    $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
     $tinfo->{'comment'}= "Test needs debug binaries";
     return $tinfo
   }
@@ -1044,6 +1053,7 @@ sub collect_one_test_case {
     {
       # ndbcluster is disabled
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "ndbcluster disabled";
       return $tinfo;
     }
@@ -1055,6 +1065,7 @@ sub collect_one_test_case {
     {
       # Only the ndb test should be run, all other should be skipped
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "Only ndbcluster tests";
       return $tinfo;
     }
@@ -1075,6 +1086,7 @@ sub collect_one_test_case {
     {
       # innodb is not supported, skip it
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       # This comment is checked for running with innodb plugin (see above),
       # please keep that in mind if changing the text.
       $tinfo->{'comment'}= "No innodb support";
@@ -1097,6 +1109,7 @@ sub collect_one_test_case {
     if (grep(/^--skip-log-bin/,  @::opt_extra_mysqld_opt) )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "Test needs binlog";
       return $tinfo;
     }
@@ -1114,6 +1127,7 @@ sub collect_one_test_case {
     if ( $skip_rpl )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "No replication tests(--skip-rpl)";
       return $tinfo;
     }
@@ -1124,6 +1138,7 @@ sub collect_one_test_case {
     if ( $tinfo->{'not_embedded'} )
     {
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "Not run for embedded server";
       return $tinfo;
     }
@@ -1135,6 +1150,7 @@ sub collect_one_test_case {
     if ( ! $::opt_ssl_supported ) {
       # SSL is not supported, skip it
       $tinfo->{'skip'}= 1;
+      $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
       $tinfo->{'comment'}= "No SSL support";
       return $tinfo;
     }
@@ -1143,6 +1159,7 @@ sub collect_one_test_case {
   if ( $tinfo->{'not_windows'} && IS_WINDOWS )
   {
     $tinfo->{'skip'}= 1;
+    $tinfo->{'skip_reason'}= MTR_SKIP_BY_FRAMEWORK;
     $tinfo->{'comment'}= "Test not supported on Windows";
     return $tinfo;
   }
