@@ -34,6 +34,7 @@
 #include "buffer.h"
 #include "audit_handler.h"
 #include "filter.h"
+#include "security_context_wrapper.h"
 
 #define PLUGIN_VERSION 0x0002
 
@@ -49,20 +50,20 @@ typedef void (*escape_buf_func_t)(const char *, size_t *, char *, size_t *);
 static audit_handler_t *log_handler= NULL;
 static ulonglong record_id= 0;
 static time_t log_file_time= 0;
-char *audit_log_file;
-char default_audit_log_file[]= "audit.log";
-ulong audit_log_policy= ALL;
-ulong audit_log_strategy= ASYNCHRONOUS;
-ulonglong audit_log_buffer_size= 1048576;
-ulonglong audit_log_rotate_on_size= 0;
-ulonglong audit_log_rotations= 0;
-char audit_log_flush= FALSE;
-ulong audit_log_format= OLD;
-ulong audit_log_handler= HANDLER_FILE;
-char *audit_log_syslog_ident;
-char default_audit_log_syslog_ident[] = "percona-audit";
-ulong audit_log_syslog_facility= 0;
-ulong audit_log_syslog_priority= 0;
+static char *audit_log_file;
+static const char default_audit_log_file[]= "audit.log";
+static ulong audit_log_policy= ALL;
+static ulong audit_log_strategy= ASYNCHRONOUS;
+static ulonglong audit_log_buffer_size= 1048576;
+static ulonglong audit_log_rotate_on_size= 0;
+static ulonglong audit_log_rotations= 0;
+static char audit_log_flush= FALSE;
+static ulong audit_log_format= OLD;
+static ulong audit_log_handler= HANDLER_FILE;
+static char *audit_log_syslog_ident;
+static const char default_audit_log_syslog_ident[] = "percona-audit";
+static ulong audit_log_syslog_facility= 0;
+static ulong audit_log_syslog_priority= 0;
 static char *audit_log_exclude_accounts= NULL;
 static char *audit_log_include_accounts= NULL;
 static char *audit_log_exclude_commands= NULL;
@@ -616,6 +617,14 @@ char *audit_log_general_record(char *buf, size_t buflen,
   query_length= my_charset_utf8mb4_general_ci.mbmaxlen *
                 event->general_query_length;
 
+  /* Note: query_length is the maximun size using utf8m4(4 bytes) that
+   * event->general_query_length may use. In the if branch, we convert it to
+   * utf8mb4. We store the recalculated (real size) length to query variable
+   * and use the remaing of the buffer for the output that will be printed to
+   * audit log. Parameter char *buf must be big enough to store
+   * the query (using utf8mb4) + the full output of audit event, which will
+   * contain the query again. At the else branch we estime this size.
+   */
   if (query_length < (size_t) (endbuf - endptr))
   {
     uint errors;
@@ -638,7 +647,7 @@ char *audit_log_general_record(char *buf, size_t buflen,
     query= escape_string(event->general_query, event->general_query_length,
                          endptr, endbuf - endptr, &endptr, &full_outlen);
     full_outlen*= my_charset_utf8mb4_general_ci.mbmaxlen;
-    full_outlen+= query_length * my_charset_utf8mb4_general_ci.mbmaxlen;
+    full_outlen+= query_length;
   }
 
   user= escape_string(event->general_user, event->general_user_length,
@@ -653,10 +662,8 @@ char *audit_log_general_record(char *buf, size_t buflen,
   db= escape_string(default_db, strlen(default_db),
                     endptr, endbuf - endptr, &endptr, &full_outlen);
 
-  buflen_estimated= full_outlen * 2 +
-                    strlen(format_string[audit_log_format]) +
-                    strlen(name) +
-                    event->general_sql_command.length +
+  buflen_estimated= full_outlen + strlen(format_string[audit_log_format]) +
+                    strlen(name) + event->general_sql_command.length +
                     20 + /* general_thread_id */
                     20 + /* status */
                     MAX_RECORD_ID_SIZE + MAX_TIMESTAMP_SIZE;
@@ -1062,7 +1069,8 @@ const char *next_word(const char *str, size_t *len,
 
 
 static
-void audit_log_update_thd_local(audit_log_thd_local *local,
+void audit_log_update_thd_local(MYSQL_THD thd,
+                                audit_log_thd_local *local,
                                 unsigned int event_class,
                                 const void *event)
 {
@@ -1076,18 +1084,21 @@ void audit_log_update_thd_local(audit_log_thd_local *local,
     const struct mysql_event_connection *event_connection=
       (const struct mysql_event_connection *) event;
 
+    const char *host = get_priv_host(thd);
+    const size_t host_length = strlen(host);
+
     local->skip_session= FALSE;
     if (audit_log_include_accounts != NULL &&
-        !audit_log_check_account_included(event_connection->user,
-                                          event_connection->user_length,
-                                          event_connection->host,
-                                          event_connection->host_length))
+        !audit_log_check_account_included(event_connection->priv_user,
+                                          event_connection->priv_user_length,
+                                          host,
+                                          host_length))
       local->skip_session= TRUE;
     if (audit_log_exclude_accounts != NULL &&
-        audit_log_check_account_excluded(event_connection->user,
-                                         event_connection->user_length,
-                                         event_connection->host,
-                                         event_connection->host_length))
+        audit_log_check_account_excluded(event_connection->priv_user,
+                                         event_connection->priv_user_length,
+                                         host,
+                                         host_length))
       local->skip_session= TRUE;
 
     if (event_connection->status == 0)
@@ -1183,7 +1194,7 @@ void audit_log_notify(MYSQL_THD thd MY_ATTRIBUTE((unused)),
   size_t len, buflen;
   audit_log_thd_local *local= get_thd_local(thd);
 
-  audit_log_update_thd_local(local, event_class, event);
+  audit_log_update_thd_local(thd, local, event_class, event);
 
   if (!is_event_class_allowed_by_policy(event_class, audit_log_policy))
     return;
@@ -1673,8 +1684,9 @@ static struct st_mysql_sys_var* audit_log_system_variables[] =
   NULL
 };
 
-char thd_local_init_buf[sizeof(audit_log_thd_local)];
+static char thd_local_init_buf[sizeof(audit_log_thd_local)];
 
+static
 void MY_ATTRIBUTE((constructor)) audit_log_so_init()
 {
   memset(thd_local_init_buf, 1, sizeof(thd_local_init_buf) - 1);
