@@ -48,6 +48,69 @@ extern bool    release_mdl_lock;
 
 extern option_t config_option_names[];
 
+cb_open_table_t			ib_cb_open_table;
+cb_read_row_t			ib_cb_read_row;
+cb_insert_row_t			ib_cb_insert_row;
+cb_cursor_delete_row_t		ib_cb_delete_row;
+cb_cursor_update_row_t		ib_cb_update_row;
+cb_cursor_moveto_t		ib_cb_moveto;
+cb_sec_search_tuple_create_t	ib_cb_search_tuple_create;
+cb_sec_read_tuple_create_t	ib_cb_read_tuple_create;
+cb_tuple_delete_t		ib_cb_tuple_delete;
+cb_tuple_copy_t			ib_cb_tuple_copy;
+cb_tuple_read_u8_t		ib_cb_tuple_read_u8;
+cb_tuple_read_u16_t		ib_cb_tuple_read_u16;
+cb_tuple_read_u32_t		ib_cb_tuple_read_u32;
+cb_tuple_read_u64_t		ib_cb_tuple_read_u64;
+cb_tuple_write_u8_t		ib_cb_tuple_write_u8;
+cb_tuple_write_u16_t		ib_cb_tuple_write_u16;
+cb_tuple_write_u32_t		ib_cb_tuple_write_u32;
+cb_tuple_write_u64_t		ib_cb_tuple_write_u64;
+cb_tuple_read_i8_t		ib_cb_tuple_read_i8;
+cb_tuple_read_i16_t		ib_cb_tuple_read_i16;
+cb_tuple_read_i32_t		ib_cb_tuple_read_i32;
+cb_tuple_read_i64_t		ib_cb_tuple_read_i64;
+cb_tuple_write_i8_t		ib_cb_tuple_write_i8;
+cb_tuple_write_i16_t		ib_cb_tuple_write_i16;
+cb_tuple_write_i32_t		ib_cb_tuple_write_i32;
+cb_tuple_write_i64_t		ib_cb_tuple_write_i64;
+cb_col_set_value_t		ib_cb_col_set_value;
+cb_col_get_value_t		ib_cb_col_get_value;
+cb_col_get_meta_t		ib_cb_col_get_meta;
+cb_trx_begin_t			ib_cb_trx_begin;
+cb_trx_commit_t			ib_cb_trx_commit;
+cb_trx_rollback_t		ib_cb_trx_rollback;
+cb_trx_start_t			ib_cb_trx_start;
+cb_trx_state_t			ib_cb_trx_state;
+cb_trx_release_t		ib_cb_trx_release;
+cb_tuple_get_n_cols_t		ib_cb_tuple_get_n_cols;
+cb_cursor_set_match_mode_t	ib_cb_cursor_set_match_mode;
+cb_cursor_lock_t		ib_cb_cursor_lock;
+cb_cursor_close_t		ib_cb_cursor_close;
+cb_cursor_new_trx_t		ib_cb_cursor_new_trx;
+cb_open_table_by_name_t		ib_cb_open_table_by_name;
+cb_cursor_reset_t		ib_cb_cursor_reset;
+cb_col_get_name_t		ib_cb_col_get_name;
+cb_get_idx_field_name		ib_cb_get_idx_field_name;
+cb_table_truncate_t		ib_cb_table_truncate;
+cb_cursor_first_t		ib_cb_cursor_first;
+cb_cursor_next_t		ib_cb_cursor_next;
+cb_cursor_last_t		ib_cb_cursor_last;
+cb_cursor_open_index_using_name_t	ib_cb_cursor_open_index_using_name;
+cb_close_thd_t			ib_cb_close_thd;
+cb_get_cfg_t			ib_cb_get_cfg;
+cb_cursor_set_memcached_sync	ib_cb_cursor_set_memcached_sync;
+cb_cursor_set_cluster_access_t	ib_cb_cursor_set_cluster_access;
+cb_cursor_commit_trx_t		ib_cb_cursor_commit_trx;
+cb_cfg_trx_level_t		ib_cb_cfg_trx_level;
+cb_get_n_user_cols		ib_cb_get_n_user_cols;
+cb_cursor_set_lock_t		ib_cb_cursor_set_lock;
+cb_cursor_clear_trx_t		ib_cb_cursor_clear_trx;
+cb_trx_get_start_time		ib_cb_trx_get_start_time;
+cb_bk_commit_interval		ib_cb_cfg_bk_commit_interval;
+cb_cursor_stmt_begin		ib_cb_cursor_stmt_begin;
+cb_trx_read_only_t		ib_cb_trx_read_only;
+
 /** InnoDB API callback functions */
 static ib_cb_t* innodb_memcached_api[] = {
 	(ib_cb_t*) &ib_cb_open_table,
