@@ -1625,7 +1625,7 @@ void do_handle_one_connection(THD *thd_arg)
   if (WSREP(thd))
   {
     mysql_mutex_lock(&thd->LOCK_wsrep_thd);
-    thd->wsrep_query_state= QUERY_EXITING;
+    wsrep_thd_set_query_state(thd, QUERY_EXITING);
     mysql_mutex_unlock(&thd->LOCK_wsrep_thd);
   }
 #endif
