@@ -230,7 +230,6 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
   {
     if ((options & REFRESH_READ_LOCK) && thd)
     {
-      bool own_lock;
       /*
         On the first hand we need write lock on the tables to be flushed,
         on the other hand we must not try to aspire a global read lock
@@ -247,7 +246,12 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
 	UNLOCK TABLES
       */
       tmp_write_to_binlog= 0;
+#ifdef WITH_WSREP
+      bool own_lock;
       if (thd->global_read_lock.lock_global_read_lock(thd, &own_lock))
+#else
+      if (thd->global_read_lock.lock_global_read_lock(thd))
+#endif
         return 1; // Killed
       if (close_cached_tables(thd, tables,
                               ((options & REFRESH_FAST) ?  FALSE : TRUE),
@@ -263,10 +267,14 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
       if (thd->global_read_lock.make_global_read_lock_block_commit(thd)) // Killed
       {
         /* Don't leave things in a half-locked state */
+#ifdef WITH_WSREP
         if (own_lock)
         {
+#endif
           thd->global_read_lock.unlock_global_read_lock(thd);
+#ifdef WITH_WSREP
         }
+#endif
         return 1;
       }
 #ifdef WITH_WSREP
