@@ -20,16 +20,20 @@
 
 int wsrep_show_bf_aborts (THD *thd, SHOW_VAR *var, char *buff);
 void wsrep_client_rollback(THD *thd);
+void wsrep_replay_sp_transaction(THD* thd);
 void wsrep_replay_transaction(THD *thd);
 void wsrep_create_appliers(long threads);
 void wsrep_create_rollbacker();
 
-extern "C" my_bool  wsrep_thd_is_BF(void *thd_ptr, my_bool sync);
-extern "C" void  wsrep_thd_set_PA_safe(void *thd_ptr, my_bool safe);
+int  wsrep_abort_thd(void *bf_thd_ptr, void *victim_thd_ptr,
+                                my_bool signal);
+
+extern void  wsrep_thd_set_PA_safe(void *thd_ptr, my_bool safe);
+extern my_bool  wsrep_thd_is_BF(void *thd_ptr, my_bool sync);
+extern int wsrep_thd_conflict_state(void *thd_ptr, my_bool sync);
+//extern "C" my_bool  wsrep_thd_is_BF(void *thd_ptr, my_bool sync);
 extern "C" my_bool  wsrep_thd_is_BF_or_commit(void *thd_ptr, my_bool sync);
 extern "C" my_bool  wsrep_thd_is_local(void *thd_ptr, my_bool sync);
-extern "C" int  wsrep_abort_thd(void *bf_thd_ptr, void *victim_thd_ptr,
-                                my_bool signal);
-extern "C" int  wsrep_thd_in_locking_session(void *thd_ptr);
+int  wsrep_thd_in_locking_session(void *thd_ptr);
 
 #endif /* WSREP_THD_H */
