@@ -94,6 +94,17 @@ wsrep_prepend_PATH (const char* path)
                  "State snapshot transfer may not be working.");
 }
 
+/**
+ * This is a wrapper around the WSREP_ON macro.  This allows this check
+ * to be called from plugins.
+ */
+extern "C" bool
+wsrep_is_wsrep_on(void)
+{
+  return WSREP_ON;
+}
+
+
 namespace wsp
 {
 
@@ -777,7 +788,15 @@ void process::terminate ()
 {
   if (pid_)
   {
+    /*
+      If we have an appropriated system call, then we try
+      to terminate entire process group:
+    */
+#if _XOPEN_SOURCE >= 500 || _DEFAULT_SOURCE || _BSD_SOURCE
+    if (killpg(pid_, SIGTERM))
+#else
     if (kill(pid_, SIGTERM))
+#endif
     {
       WSREP_WARN("Unable to terminate process: %s: %d (%s)",
                  str_, errno, strerror(errno));
@@ -925,3 +944,18 @@ size_t wsrep_guess_ip (char* buf, size_t buf_len)
 
   return 0;
 }
+
+/* returns the length of the host part of the address string */
+size_t wsrep_host_len(const char* const addr, size_t const addr_len)
+{
+  // check for IPv6 notation first
+  const char* const bracket= ('[' == addr[0] ? strchr(addr, ']') : NULL);
+
+  if (bracket) { // IPv6
+    return (bracket - addr + 1);
+  }
+  else { // host part ends at ':' or end of string
+    const char* const colon= strchr(addr, ':');
+    return (colon ? colon - addr : addr_len);
+  }
+}
