@@ -1824,9 +1824,7 @@ trx_commit_or_rollback_prepare(
 			if (UNIV_UNLIKELY(trx->take_stats)) {
 				ut_usectime(&sec, &ms);
 				now = (ib_uint64_t)sec * 1000000 + ms;
-				trx->lock_que_wait_timer
-					+= (ulint)
-					(now - trx->lock_que_wait_ustarted);
+				trx->lock_que_wait_timer += now - trx->lock_que_wait_ustarted;
 			}
 
 			trx->lock.que_state = TRX_QUE_RUNNING;
